package cz.muni.fi.pa165.controller;
import cz.muni.fi.pa165.model.TeamMemberDTO;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RequestMapping("/api/v1/team-members")
@OpenAPIDefinition(
        info = @Info(title = "PA 165 Team Member Service API", version = "1.0",
                description = "API used for performing CRUD operations on Team Member"),
        servers = {
                @Server(url = "/", description = "Default Server URL")
        }
)
@Tag(name = "PA165 Team Member Service", description = "APIs for managing team members")
public interface TeamMemberController {
    @PostMapping
    @Operation(summary = "Create new team member")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<TeamMemberDTO> createTeamMember(@RequestBody @Valid TeamMemberDTO teamMemberDTO);

    @GetMapping("/{id}")
    @Operation(summary = "Get team member by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<TeamMemberDTO> getTeamMemberById(@PathVariable("id") long id);

    @PutMapping("/{id}")
    @Operation(summary = "Update existing team member by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<TeamMemberDTO> updateTeamMemberById(@PathVariable("id") long id, @RequestBody @Valid TeamMemberDTO teamMemberDTO);

    @DeleteMapping("/{id}")
    @Operation(summary = "Remove existing team member by ID")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    ResponseEntity<Void> removeTeamMemberById(@PathVariable("id") long id);

}
