package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.entity.TeamMember;
import cz.muni.fi.pa165.data.respository.TeamMemberRepository;
import cz.muni.fi.pa165.exception.TeamMemberNotFoundException;
import cz.muni.fi.pa165.mapper.TeamMemberMapper;
import cz.muni.fi.pa165.model.TeamMemberDTO;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

public class TeamMemberServiceImplTest {

    @Mock
    private TeamMemberRepository teamMemberRepository;

    @Mock
    private TeamMemberMapper teamMemberMapper;

    @Mock
    private RestTemplate restTemplate;

    @InjectMocks
    private TeamMemberServiceImpl teamMemberService;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.openMocks(this);
    }

    @Test
    public void getTeamMemberByID_whenTeamMemberExists_returnsTeamMemberDTO() {
        long id = 1;
        TeamMember teamMember = new TeamMember();
        teamMember.setId(id);
        TeamMemberDTO expected = new TeamMemberDTO();
        expected.setId(id);

        when(teamMemberRepository.findById(id)).thenReturn(Optional.of(teamMember));
        when(teamMemberMapper.teamMemberToTeamMemberDTO(teamMember)).thenReturn(expected);

        TeamMemberDTO actual = teamMemberService.getTeamMemberByID(id);

        assertEquals(expected, actual);
    }

    @Test
    public void getTeamMemberByID_whenTeamMemberDoesNotExist_throwsTeamMemberNotFoundException() {
        long id = 1;

        when(teamMemberRepository.findById(id)).thenReturn(Optional.empty());

        assertThrows(TeamMemberNotFoundException.class, () -> {
            teamMemberService.getTeamMemberByID(id);
        });
    }

    /*@Test
    public void createTeamMember_returnsTeamMemberDTO() {
        TeamMemberDTO teamMemberDTO = new TeamMemberDTO();
        TeamMember teamMember = new TeamMember();

        when(teamMemberMapper.teamMemberDTOToTeamMember(teamMemberDTO)).thenReturn(teamMember);
        when(teamMemberRepository.save(teamMember)).thenReturn(teamMember);
        when(teamMemberMapper.teamMemberToTeamMemberDTO(teamMember)).thenReturn(teamMemberDTO);
        when(restTemplate.getForEntity(anyString(), Mockito.any(), anyLong())).thenReturn(ResponseEntity.ok(null));

        TeamMemberDTO actual = teamMemberService.createTeamMember(teamMemberDTO);

        assertEquals(teamMemberDTO, actual);
    }*/

    @Test
    public void removeTeamMemberByID_whenTeamMemberExists_deletesTeamMember() {
        long id = 1;
        TeamMember teamMember = new TeamMember();
        teamMember.setId(id);

        when(teamMemberRepository.findById(id)).thenReturn(Optional.of(teamMember));

        teamMemberService.removeTeamMemberByID(id);

        Mockito.verify(teamMemberRepository).delete(teamMember);
    }

    @Test
    public void removeTeamMemberByID_whenTeamMemberDoesNotExist_throwsTeamMemberNotFoundException() {
        long id = 1;

        when(teamMemberRepository.findById(id)).thenReturn(Optional.empty());

        assertThrows(TeamMemberNotFoundException.class, () -> {
            teamMemberService.removeTeamMemberByID(id);
        });
    }

    @Test
    public void updateTeamMemberByID_returnsTeamMemberDTO() {
        TeamMemberDTO teamMemberDTO = new TeamMemberDTO();
        TeamMember teamMember = new TeamMember();

        when(teamMemberMapper.teamMemberDTOToTeamMember(teamMemberDTO)).thenReturn(teamMember);
        when(teamMemberRepository.save(teamMember)).thenReturn(teamMember);
        when(teamMemberMapper.teamMemberToTeamMemberDTO(teamMember)).thenReturn(teamMemberDTO);

        TeamMemberDTO actual = teamMemberService.updateTeamMemberByID(teamMemberDTO);

        assertEquals(teamMemberDTO, actual);
    }
}